Package: Rmagpie
Version: 1.66.0
Title: MicroArray Gene-expression-based Program In Error rate
        estimation
Author: Camille Maumet <Rmagpie@gmail.com>, with
    contributions from C. Ambroise
	J. Zhu
Maintainer: Camille Maumet <Rmagpie@gmail.com>
Depends: R (>= 2.6.1), Biobase (>= 2.5.5)
Imports: Biobase (>= 2.5.5), e1071, graphics, grDevices, kernlab,
        methods, pamr, stats, utils
Suggests: xtable
Description: Microarray Classification is designed for both biologists and statisticians. 
    It offers the ability to train a classifier on a labelled microarray dataset and to then use
    that classifier to predict the class of new observations. A range of modern classifiers are available, including
    support vector machines (SVMs), nearest shrunken centroids (NSCs)... Advanced methods are provided to estimate
    the predictive error rate and to report the  subset of genes which appear essential in discriminating between classes.
License: GPL (>= 3)
LazyLoad: yes
URL: http://www.bioconductor.org/
biocViews: Microarray, Classification
collate: AllClasses.R AllGenerics.R AllInitialize.R
        classifyNewSamples.R convertDataFile.R assessment-accessors.R
        featureSelectionOptions-accessors.R finalClassifier-accessors.R
        geneSubsets-accessors.R initialize-methods.R loadData-methods.R
        non_linear_svm.R one_layer_ext_CV.R rateGenes.R
        result2LayerCV-accessors.R resultRepeated1LayerCV-accessors.R
        rfe_zhu.R runOneLayerExtCV-methods.R runTwoLayerExtCV-methods.R
        setValidity-methods.R show-methods.R thresholds-accessors.R
        tool_case.R two_layer_ext_CV.R typeValidity.R user_interface.R
git_url: https://git.bioconductor.org/packages/Rmagpie
git_branch: RELEASE_3_22
git_last_commit: de6a8a5
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 06:05:33 UTC; biocbuild
